/* 
 * blink.c: blinks an LED
 *   The program makes use of the pigpio library
 *    Copyright U. Raich Dec. 2016
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <pigpio.h>
#include <stdbool.h>
#include <errno.h>
#include <fcntl.h>

#define LED_PIN 17
#define OFF     0
#define ON      1

void blinkExit(int dummy) {
  /*
    clean everything and exit 
  */
  printf("blink: Cleaning up\n");
  gpioWrite(LED_PIN,OFF);
  gpioTerminate();
  exit(0);
}

int main (int arrgc, char ** argv) {
  int errCode;
  
  /*
    initilize the pigpio library
  */
  if ((errCode=gpioInitialise()) < 0)
    {
      fprintf(stderr,"blink.c: Could not initialize the pigpio library\n");
      fprintf(stderr,"error code was: %d\n",errCode);
      return -1;
    }
  else
    {
      printf("blink.c: pigpio library successfully initialized\n");
    }
  /* 
     call this function just before we exit 
  */
  signal(SIGINT,blinkExit);
  /*
    set gpio 17 to output
  */
  if ((errCode=gpioSetMode(LED_PIN, PI_OUTPUT)) <0)
    {
      fprintf(stderr,"blink.c: Could not set mode to output for pin %d\n",
	      LED_PIN);
      fprintf(stderr,"error code was: %d\n",errCode);
      return -1;
    }
  else
    {
      printf("blink.c: Pin %d sucessfully set to output\n",LED_PIN);
    }
  while(true) {
    if ((errCode=gpioWrite(LED_PIN,ON)) < 0) {
      fprintf(stderr,"blink.c: Could not switch LED \"on\" on pin %d\n",LED_PIN);
      fprintf(stderr,"error code was: %d\n",errCode);
      return -1;
    }
    else {
      if((errCode=gpioSleep(PI_TIME_RELATIVE,1,0))<0) {
	fprintf(stderr,"blink.c: Error in gpioSleep: %d\n",errCode);
	return -1;
      }
      gpioWrite(LED_PIN,OFF);
      gpioSleep(PI_TIME_RELATIVE,1,0);
    }
  }
  return 0;
}
