/*
 * blink2.c
 *    This program uses the socket interface to pigpiod to make the LED blink
 *    Copyright U. Raich Dec. 2016
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2017
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <stdbool.h>
#include <pigpiod_if2.h>
#include "pcf8591.h"

#define LED_PIN          17
#define OFF              0
#define ON               1

#define I2C_BUS          1
#define I2C_FLAGS        0
#define PCF8591_ADDRESS  0x48

int pi=-1,handle=-1;
void heartbeatExit(int dummy) {
  /*
    clean everything and exit 
  */
  printf("heartbeat: Cleaning up\n");
  gpio_write(pi, LED_PIN,OFF);
  pigpio_stop(pi);
  exit(0);
}


int main(int argc, char **argv) {
  int i,retCode;
  unsigned char controlByte,adcValue;
  if ((pi=pigpio_start(NULL,NULL)) < 0) {
    printf("heartbeat: Could not open connection to pigpiod\n");
    printf("%s\n",pigpio_error(pi));
    return -1;
  }
  printf("blink2: Connection to pigpiod successfully opened\n");

  if ((retCode=set_mode(pi,LED_PIN,PI_OUTPUT)) < 0) {
    printf("heatbeat: Could not set LED pin to output\n");
    printf("%s\n",pigpio_error(retCode));
    return -1;
  }
  /*
    capture ^C to gracefully exit the program
  */
  signal(SIGINT,heartbeatExit);

  /*
    open the I2C bus
  */
  if ((retCode = i2c_open(pi,I2C_BUS,PCF8591_ADDRESS,I2C_FLAGS)) < 0) {
    printf("pcf8951: Cannot open I2C bus %d at address %02x\n",
	   I2C_BUS,PCF8591_ADDRESS);
    if ((pigifError_t)retCode == pigif_bad_send) {
      printf("Did you start the pigpiod daemon?\n");
      return -1;
    }
  }
  handle = retCode;

  controlByte= PCF8591_CHANNEL_C | PCF8591_4_SINGLE_ENDED;
  retCode = i2c_write_byte(pi, handle, controlByte);
  if (retCode < 0) {
    printf("pcf8951; Cannot write control byte 0x%02x to address 0x%02x\n",
	   controlByte,PCF8591_ADDRESS);
    return -1;
  }
  /*
    switch the IR LED on
  */
     printf("heartbeat: switching LED on\n");
    if ((retCode=gpio_write(pi, LED_PIN,ON))<0) {
      printf("heartbeat: Could not set LED pin to output\n");
      printf("%s\n",pigpio_error(retCode));
      return -1;
    }
 
  while (true) {

    for(i=0;i<10;i++) {
      adcValue = i2c_read_byte(pi, handle);
      printf("%d\n",adcValue);
      time_sleep(0.1);
    }
  } 
}

